#!/usr/bin/perl

use Cwd;

# Print the final table for given hash argument
sub print_table {
  my $hash = shift;
  foreach my $key (sort keys $hash) {
    print "\n\n# Results for C '$key' glob:\n";
    print "-"x40 . "\n";
    printf("%10s | %10s\n", 'noescape', 'Perl regular expression');
    print "-"x40 . "\n";
    printf("%10s | %10s\n", 'FALSE', $hash->{$key}->{'FALSE'});
    printf("%10s | %10s\n", 'TRUE', $hash->{$key}->{'TRUE'});
    print "-"x40 . "\n";
  }
}

# Convert provided glob to Perl regular expression based on 'noescape' value
sub glob2pat {
    my ($globstr, $noescape) = (shift, shift);
    my $origglob = $globstr;
    my %patmap = (
        '*' => '.*',
        '?' => '.',
        '[' => '[',
        ']' => ']',
    );
    if ($noescape) {
      # treat '\' as literal
      $globstr =~ s{(.)} {$patmap{$1} || "\Q$1"}ge;
      print "- noescape = TRUE:";
      print "\n\tC glob: '$origglob' => Perl regex: '" . '^' . $globstr . '$' . "'";
    }
    else {
      # treat '\' as escape character
      $globstr =~ s{(.)} {$patmap{$1} || "$1"}ge;
      print "- noescape = FALSE:";
      print "\n\tC glob: '$origglob' => Perl regex: '" . '^' . $globstr . '$' . "'";
    }
    
    return '^' . $globstr . '$';
}

# noescape = false case should be the default
my $noescape = 0;

# Retrieve list of file names in CWD
my @files = glob("*");
# Including '.' directory for '\*' and '\?' pattern cases
push(@files, '.');

# Define list of globs to be converted here 
my @globs = ('\*', '\?', '\[hello\]');

# Define hash to store the table results
my %table = ();

foreach my $glob (@globs) {
  $noescape = 0; # noescape = false case
  my $pat = glob2pat($glob, $noescape);
  $table{$glob}{'FALSE'} = $pat;
  print "\n\tList of files in CWD matching Perl regex:\n\t";
  foreach my $file (@files) {
    if ($file =~ /$pat/) {
      print " '$file'";
    }
  }
  print "\n";
  $noescape = 1; # noescape = true case
  $pat = glob2pat($glob, $noescape);
  $table{$glob}{'TRUE'} = $pat;
  print "\n\tList of files in CWD matching Perl regex:\n\t";
  foreach my $file (@files) {
    if ($file =~ /$pat/) {
      print " '$file'";
    }
  }
  print "\n";
}

# Print the table
print_table(\%table);
